
#include <string.h>
#include <io.h>
#include <stdio.h>
#include <fcntl.h>
#include <dir.h>
#include <dos.h>
#include <stdarg.h>

#define FILENAMELENGTH 13
#define MAXFILES 100

#define YES 1
#define NO  0


int pauseflag=NO;
char filetable[MAXFILES][FILENAMELENGTH];
static char copywrong[]="NDX1 Copywrong R. Russell Freeland version 1.0";
static char whuzzup[]="What are you doing in here, anyway?";

/* waitkey() is like INKEY(0), it always waits for a key to be struck.  */
/* geninterrupt() is not a function, rather it is a macro to the Turbo  */
/* C preprocessor.  In the following function, geninterrupt(0x16) is    */
/* compiled to one assembler instruction: INT 16h                       */

void waitkey(void) {
      _AH = 0;                      /* service 0 of BIOS interrupt 16h, */
      geninterrupt(0x16);           /* wait until key struck            */
}

/************************************************************************/
/*  pauseprintf(): keep a line count and pause if flagged;              */
/*                 pass params to printf() in any case                  */
/*                                                                      */
/*                                                                      */
/*  This function uses the "va_list" syntax of Turbo C to pass a        */
/*  variable number of parameters down to another function (printf()).  */
/*                                                                      */
/************************************************************************/

#define PAGESIZE 23

void pauseprintf(char *format,...) {
    va_list argptr;
    static int line_count=0;
    char text[200];               /* arbitrary number, sure to screw up */

    va_start(argptr,format);      /* using this to pass variable        */
    vsprintf(text,format,argptr); /* # of parameters to pauseprintf()   */
    va_end(argptr);
    printf("\n%s",text);
    if (!pauseflag) return;
    line_count++;
    if (line_count>=PAGESIZE) {
        line_count=0;
        printf("\n--more--");
        waitkey();
    }
}


/************************************************************************/
/*  justpath(): isolate just the path portion of a DOS pathname         */
/*              Strategy: look first for "\", then ":", using the C     */
/*              runtime library's strrchr() function, which is similar  */
/*              to the Clipperites' old favorite RAT() ("reverse AT()") */
/************************************************************************/


int justpath(char *paramname) {
    char *mark;

    mark=strrchr(paramname,'\\');
    if (mark!=NULL) *(mark+1)=0;
     else if ((mark=strchr(paramname,':'))!=NULL) *(mark+1)=0;
      else *paramname=0;
    return(strlen(paramname));
}

/************************************************************************/
/*  getfullpathname(): expand a filename to include full drive/path     */
/*              ASSUMES THE PASSED STRING IS LONG ENOUGH!!!             */
/*              (requires DOS 3, not sure what happens otherwise)       */
/************************************************************************/

void getfullpathname(char *s) {
    char *mark;

    _ES=_DS=FP_SEG(s);                 /* we'll use the same string for */
                                       /* both source and target, so    */
    _SI=_DI=FP_OFF(s);                 /* the segment address goes in   */
                                       /* ES & DS, offset in DI & SI    */
    _AH=0x60;                          /* setup service 60h of INT 21   */
    geninterrupt(0x21);                /* call DOS                      */
    mark=strstr(s,"????????");         /* DOS returns "????????" instead*/
                                       /* of "*"                        */
    if (mark!=NULL) {
        *mark='*';
        strcpy(mark+1,mark+8);
    }
}

/************************************************************************/
/*  getfiles(): expand "wildcard" into full path name, find all         */
/*              matching files and stuff 'em into filetable[],          */
/*              then return the count of files found                    */
/************************************************************************/

int getfiles(char *pwildcard, char *ext) {
    char wildcard[128];
    int done;
    int fcount=0;
    int prevcount=fcount;
    struct ffblk dta;                  /* Turbo C, for findfirst()      */
    char *mark,*fname;

    strcpy(wildcard,pwildcard);        /* make sure the string we pass  */
                                       /* to getfullpath()name  is long */
                                       /* enough
                                                      */
    if ((!strpbrk(wildcard,"\*\?")) && findfirst(wildcard,&dta,16)==0) {
        if (dta.ff_attrib==16) {       /* if param was a directory name */
              strcat(wildcard,"\\");   /* add a backslash               */
              strcat(pwildcard,"\\");  /* to both local and param names */
        }
    }
    fname=(strrchr(wildcard,'\\'));    /* look for end of path          */
    if (fname==NULL)                   /* if not found then             */
          fname=strrchr(wildcard,':'); /* look for drive (colon)        */
    if (fname==NULL) fname=wildcard;   /* if no drive/path assign wcard */
     else fname++;                     /* else move past colon/bkslash  */
    if (strlen(fname)==0)              /* if only a drive/path name,    */
          strcpy(fname,"*");           /* supply the wildcard "*"       */
    mark=strchr(fname,'.');
    if (mark==NULL) {                  /* if no "." in name             */
        strcat(wildcard,".");          /* add ".*"                      */
        strcat(wildcard,ext);
    }
    done=(findfirst(wildcard,&dta,39));/* find any file but a subdir,   */
                                       /* add 1=R/O, 2=hidden,          */
                                       /* 4=system, 32=archive          */
    if (done) {
        pauseprintf("%s: No matching files found!",wildcard);
        return(0);
    }
    while (!done) {
        strcpy(filetable[fcount],dta.ff_name);
                                       /* collect the files so we       */
        fcount++;                      /* can sort them                 */
        if (fcount==MAXFILES) {
              pauseprintf("%d (maximum) files exceeded!",MAXFILES);
              break;
        }
        done=findnext(&dta);
    }
  /* if we found files matching spec, print count and sort alphabetically */
    if (fcount>1) {
        qsort(filetable,fcount,FILENAMELENGTH,strcmp);
        getfullpathname(wildcard);
        pauseprintf("%d files matching %s:",fcount-prevcount,wildcard);
    }
    return(fcount);
}

/************************************************************************/
/*  syntaxerror(): slap user's wrist and exit with DOS errorlevel 1     */
/************************************************************************/

void syntaxerror(void) {
    printf("\n\7Syntax: NDX1 [filespec] [/P]");
    printf("\nDumps NDX/MDX header structure(s) to the screen");
    printf("\nSee Data Based Advisor July 1990 for details");
    exit(1);
}

/************************************************************************/
/*  isexpr(): see if pointer points to a string which looks like it     */
/*            might be a valid index expression.  Critera: expression   */
/*            must be less than 200 bytes long (arbitrary number for    */
/*            now) and at least one byte long (not including the null), */
/*            and must be comprised of ASCII characters between " "     */
/*            and "}".                                                  */
/************************************************************************/

int isexpr(char *s) {
    int i=0;

    while (i++<200) {
        if ( (*s < ' ') || (*s > '}') ) return(0);
        if (*s==0) {
            if (i>2) return(1);
            return(0);
        }
    }
}

char scrapheap[1024];                  /* where to read the headers    */

#define UNRECOGNIZED -1
#define II 0
#define FOXPLUS 1
#define CLIPPER 2
#define III 3
#define MDX 4

static char *ndxtypenames[5]={
    "dBASE II",
    "FoxBASE+/Pro",
    "Clipper",
    "dBASE III/+",
    "dBASE IV MDX"
};

static int offsets[4]={10,16,22,24};    /* starting offsets for index   */
                                        /* expression strings for first */
                                        /* 4 index types                */
main(int argc, char *argv[]) {
    char pathname[128],*sumarg,*fname;
    int ndxversion;
    int counter,numfiles,pathlen,fhandle;

    _fmode=O_BINARY;
    if (argc>3) syntaxerror();
    for (counter=1;counter<3;counter++) {
        sumarg=argv[counter];
        if (*sumarg=='/') {
            ++sumarg;
            if (*sumarg=='p' || *sumarg=='P') {
                  if (counter==1) argv[1]=argv[2];
                  /* setup filename in first argument */
                  pauseflag=YES;
            }
        }
    }
    if (argc < 2) strcpy(pathname,"*"); /* if no arguments supply one   */
     else strcpy(pathname,argv[1]);     /* setup local (longer) var     */

    numfiles=getfiles(pathname,"??X");  /* build sorted table of files  */
    pathlen=justpath(pathname);         /* save length of path portion  */
    if (numfiles>0) {
        pauseprintf("File name    Index type   Expression");
        pauseprintf("===============================================================================");
    }
    for (counter=0;counter<numfiles;counter++) {
        fname=filetable[counter];
        strcpy(pathname+pathlen,fname);

        if ((fhandle = open(pathname,O_RDONLY+O_BINARY)) == -1) {
            pauseprintf("\7%-13s: Can't open file!",fname);
            continue;
        }

        if (_read(fhandle,scrapheap,1024)<10) {
            pauseprintf("\7%-13s: Can't read header!",fname);
            _close(fhandle);
            continue;
        }
        _close(fhandle);

        ndxversion=UNRECOGNIZED;
        if (scrapheap[0]==2) {          /* flag for dBASE IV MDX        */
            pauseprintf("%-13s  !! dBASE IV MDX!  Next month's version",fname);
            continue;
        }

        /* look for valid index expression in header at offsets we know */
        /* should be made generic to work with "offsets" array          */

        if (isexpr(scrapheap+10)) ndxversion=II;
         else if (isexpr(scrapheap+16)) ndxversion=FOXPLUS;
          else if (isexpr(scrapheap+22)) ndxversion=CLIPPER;
           else if (isexpr(scrapheap+24)) ndxversion=III;

        if (ndxversion==UNRECOGNIZED) {
            pauseprintf("\7%-13s  !Unrecognized index file type!",fname);
            continue;
        }
        pauseprintf("%-13s%-13s%s",fname,ndxtypenames[ndxversion],
                        scrapheap+offsets[ndxversion]);
    }

}

